# Update get_iplayer to new version

import subprocess
import os
from shutil import copyfile
import requests


def replaceInTag(text, tag_name, new_inner):
    start_tag, end_tag = '<' + tag_name + '>', '</' + tag_name + '>'
    start_pos, end_pos = text.find(start_tag), nuspec.find(end_tag)

    text = text[:start_pos] + start_tag + new_inner + text[end_pos:]
    return text


# folders
user_dir = os.environ['USERPROFILE']
app_name = 'get_iplayer'
choco_name = 'getiplayer'
get_iplayer_folder = os.path.join(user_dir, 'GitHub', app_name)
wiki_folder = get_iplayer_folder + '_wiki'
choco_folder = os.path.join(user_dir, 'Documents', 'Scripts', choco_name)

# update GitHub + wiki folders (I think 'fetch' is correct)
assert subprocess.call('git fetch', cwd=get_iplayer_folder) == 0
assert subprocess.call('git fetch', cwd=wiki_folder) == 0

nuspec_filename = os.path.join(choco_folder, choco_name + '.nuspec')
nuspec = open(nuspec_filename, encoding='utf-8').read()

authors = open(os.path.join(get_iplayer_folder, 'CONTRIBUTORS')).read().replace('\n', ', ')[:-2]
nuspec = replaceInTag(nuspec, 'authors', authors)

release_notes = open(os.path.join(wiki_folder, 'releasenotes.md')).read().splitlines()

# find first link - should point to the newest version
for line in release_notes:
    start_pos, mid_pos, end_pos = line.find('['), line.find(']('), line.find(')')
    if -1 in (start_pos, mid_pos, end_pos):
        continue
    link_name = line[start_pos+1:mid_pos]
    link_dest = line[mid_pos+2:end_pos]
    if link_name.startswith(app_name):
        version = link_name.split(' ')[1]
        break

print(app_name + ' version ' + version)
nuspec = replaceInTag(nuspec, 'version', version)

# get info from readme
readme = open(os.path.join(get_iplayer_folder, 'README.md')).read().splitlines()

# get first two sections (title and 'Features')
description = ''
sections = 0
for line in readme:
    if line.startswith('## '):
        sections += 1
        if sections > 2:
            break
    description += line.replace('<', '`').replace('>', '`') + '\n'

nuspec = replaceInTag(nuspec, 'description', description)

release_notes_file, link_name = link_dest.split('#')
release_notes = open(os.path.join(wiki_folder, release_notes_file + '.md'), encoding='utf-8').read().splitlines()

new_release_notes = ''
in_section = False
for line in release_notes:
    if line.startswith('<a name="' + link_name + '"/>'):
        in_section = True
        continue
    if in_section:
        if line.startswith('<a name="'):  # next section
            break
        new_release_notes += line.replace('<', '`').replace('>', '`') + '\n'  # < and > need to be escaped

print(new_release_notes)
nuspec = replaceInTag(nuspec, 'releaseNotes', new_release_notes)
copyfile(nuspec_filename, nuspec_filename + '.bak')
open(nuspec_filename, 'w', encoding='utf-8').write(nuspec)

# get binary info
json = requests.get("https://api.github.com/repos/get-iplayer/get_iplayer_win32/releases/latest").content
release = eval(json.replace(b'false', b'False').replace(b'null', b'None').replace(b'true', b'True'))

for asset in release['assets']:
    name = asset['name']
    url = asset['browser_download_url']
    if name.endswith('.exe'):
        exe_url = url
    elif name.endswith('.sha1'):
        sha1 = requests.get(url).content.decode('utf-8').split(' ')[0]
    elif name.endswith('.sha256'):
        sha256 = requests.get(url).content.decode('utf-8').split(' ')[0]

# update the exe URL and checksums in the install file
install_file = os.path.join(choco_folder, 'tools', 'chocolateyinstall.ps1')
install = open(install_file, encoding='utf-8').read().splitlines()
new_install = ''
for line in install:
    pos = line.find("'")
    if line.startswith('$url'):
        line = line[:pos] + "'" + exe_url + "'"
    elif 'checksum ' in line:
        line = line[:pos] + "'" + sha1 + "'"
    elif 'checksum64' in line:
        line = line[:pos] + "'" + sha256 + "'"
    new_install += line + '\n'

open(install_file, 'w', encoding='utf-8').write(new_install)

# package and push to server
assert subprocess.call('choco pack', cwd=choco_folder) == 0
assert subprocess.call('choco push', cwd=choco_folder) == 0
